;; I have written a little extension to the current cc-mode which adds
;; *automatically* block-close-comments.
;; 
;; What are block-close-comments?
;; 
;; Look at this example:
;; 
;; ,----
;; | for (i = 1; i < MAX_VALUE; i++)
;; | {
;; |     // do here some code
;; |     // ...
;; |     // ...
;; |     
;; | } // for (i = 1; i < MAX_VALUE; i++)
;; `----
;; 
;; Suppose the for-block in this example is larger than - lets say - 15 or
;; more
;; lines then it is good programming style to add the block-opening statement
;; behind the block closing brace. It increases the readability of code a lot.
;; Of course this comment is superfluous if the block is very small.
;; 
;; With my cc-mode extension such a block-close-comment will be inserted
;; *automatically* after hitting } if the block-size is larger than a
;; *customizable* value and if the block is not a do-while-loop.
;; 
;; What must you do to activate this extension?
;; 1. The c-hanging-braces-alist must contain '(block-close . my-c-snug-do-while)
;;    E.g.
;;    (c-hanging-braces-alist . ((substatement-open . (before after))
;;                               (brace-list-open)
;;                               (brace-list-close)
;;                               (brace-list-intro)
;;                               (brace-entry-open)
;;                               (block-close . my-c-snug-do-while)))
;; 
;; 2. Add to your c-mode-common-hook the following:
;;    (add-hook 'c-special-indent-hook 'comment-block-close)
;; 
;; Tested with cc-mode version >= 5.25
;; 
;; Klaus
;; 
;; P.S.  here comes the code...


(defcustom c-block-close-min-size-to-comment 10
  "*Define how many lines a block at least must contain so the
block-close-comment will be added."
  :type 'integer
  :group 'c)


;; The communication-channel between `my-c-snug-do-while' and
;; `comment-block-close': `block-close-data' is set during the call of the
;; ACTION-function `my-c-snug-do-while' and evaluated within the
;; `c-special-indent-hook' (here `comment-block-close', s.b.)
;; If not nil it is a list of the following elements:
;; 1. The text of the comment to be inserted behind the closing brace of
;;    the block.
;; 2. The exact position at which the closing brace was inserted
;; 3. The size of the block in lines
(defvar block-close-data nil)

(defun comment-block-close ()
  "For all blocks unless do-while-blocks this function adds after the
block-close-brace the block-begin-statement as comment, for example:
for \(i = 1; i < 6; i++\)
\{
    cout << i;
    ...
\} // for \(i = 1; i < 6; i++\)
Already hitting the '\}' inserts the comment! This works only if the
autonewline-feature of cc-mode is turned on!"
  (if (and block-close-data
           (> (nth 2 block-close-data) c-block-close-min-size-to-comment))
      ;; Because this function is hooked to c-special-indent-hook and
      ;; therefore called after EVERY indentation, we do the
      ;; comment-insert only if we have a block and its not a do-while and
      ;; block is larger the 'block-close-min-size-to-comment' lines
      (save-excursion
        ;; frirst we must do some stuff to go to the right position for the
        ;; block-close-comment:
        (goto-char (nth 1 block-close-data))
        (if (not (looking-at "[ \t]*}"))
            (next-line 1))
        (end-of-line)
        (forward-sexp -1)
        (forward-sexp 1)
        ;; now we are on the right position to insert the comment
        (when (not (looking-at (concat "[ \t]*" comment-start)))
          ;; only inserting the comment if not already one exists
          (insert (concat " "
                          comment-start
                          (progn
                            ;; remove the open { from the comment if there
                            ;; is one
                            (string-match "^\\([^{]+\\){?"
                                          (nth 0 block-close-data))
                            (match-string 1 (nth 0 block-close-data)))
                          comment-end)))))
  ;; Because this function is called after EVERY (not only in combination
  ;; with block-closes!) indentation, we must set this variable
  ;; always to nil at the end!
  (setq block-close-data nil))
  
;; like c-snug-do-while but in addition this one sets the variable
;; block-close-data (s.a.) appropriate. Only during the call of an
;; ACTION-function we have the information about the syntactic symbol of
;; current line (argument 'syntax') and the insert-position of the
;; block-close-symbol ((argument 'pos'). But we cant insert any text
;; within an ACTION function, so we must do this outside in the
;; c-special-indent-hook called after EVERY indentation. To supply this
;; hook with the necessary information we set the variable
;; block-close-data (s.a.) within the ACTION function.  This code is a only
;; slightly modified version of the original c-snug-do-while and works
;; exactly like the original one!
(defun my-c-snug-do-while (syntax pos)
  (save-excursion
    (let ((curr-point (point))
          langelem
          bc-comment-text
          bc-brace-inserted-pos
          bc-blocksize)
      (if (and (eq syntax 'block-close)
               (setq langelem (assq 'block-close c-syntactic-context))
               (progn (goto-char (cdr langelem))
                      (setq bc-blocksize (count-lines curr-point (point)))
                      (if (= (following-char) ?{)
                          (forward-sexp -1))
                      (if (looking-at "\\<do\\>[^_]")
                          ;; set block-close-data to nil but return t
                          ;; because we are in a do-while-loop
                          (not (setq block-close-data nil))
                        (end-of-line)
                        (let ((end (point)))
                          (beginning-of-line)
                          (skip-chars-forward " \t")
                          (setq bc-comment-text
                                (buffer-substring (point) end)))
                        (setq bc-brace-inserted-pos pos)
                        ;; set block-close-data to the computed data-list
                        ;; but return nil because we are NOT in a
                        ;; do-while-loop. 
                        (not (setq block-close-data
                                   `(,bc-comment-text
                                     ,bc-brace-inserted-pos
                                     ,bc-blocksize))))))
          ;; the if-condition is only t if we are in a do-while-loop
          '(before)
        '(before after)))))

(provide 'cc-mode-block-comment)
